"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeleteDomain = exports.SaveDomain = exports.GetInstanceByDomain = exports.GetMainDomain = exports.GetDomain = exports.GetDomains = exports.DomainMSSql = void 0;
var log4js = require("log4js");
var sql = require('mssql');
var logger = log4js.getLogger('Sql');
var sqlConn;
function DomainMSSql(app) {
    sqlConn = app.get('sqlConn');
}
exports.DomainMSSql = DomainMSSql;
function GetDomains(vcc, search_criteria, search_expression) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('SearchCriteria', sql.VarChar(50), search_criteria)
        .input('SearchExpression', sql.VarChar(50), search_expression)
        .execute('GetDomains')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        return Promise.resolve(recordset);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDomains = GetDomains;
function GetDomain(vcc, domain) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Domain', sql.VarChar(50), domain)
        .execute('GetDomain')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var domain = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(domain);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetDomain = GetDomain;
function GetMainDomain(vcc) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .execute('GetMainDomain')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var domain = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(domain);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetMainDomain = GetMainDomain;
function GetInstanceByDomain(domain) {
    return new sql.Request(sqlConn)
        .input('Domain', sql.VarChar(50), domain)
        .execute('GetInstanceByDomain')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var record = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve((record && record.vcc) || null);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetInstanceByDomain = GetInstanceByDomain;
function SaveDomain(vcc, domain, description, client, hasHttps, contentPort, prefix, isMainDomain, isContentDomain, contentDefaultRedirect, isDistributedService, associatedDistributedService, customCode, createdDate, createdByUserId, lastModifiedDate, lastModifiedByUserId) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Domain', sql.VarChar(50), domain)
        .input('Description', sql.VarChar(200), description)
        .input('Client', sql.VarChar(50), client)
        .input('HasHttps', sql.Bit, hasHttps)
        .input('ContentPort', sql.VarChar(50), contentPort)
        .input('Prefix', sql.VarChar(100), prefix)
        .input('IsMainDomain', sql.Bit, isMainDomain)
        .input('IsContentDomain', sql.Bit, isContentDomain)
        .input('ContentDefaultRedirect', sql.VarChar(50), contentDefaultRedirect)
        .input('IsDistributedService', sql.Bit, isDistributedService)
        .input('AssociatedDistributedService', sql.VarChar(50), associatedDistributedService)
        .input('CustomCode', sql.VarChar(sql.MAX), customCode)
        .input('CreatedDate', sql.DateTime, createdDate)
        .input('CreatedByUserId', sql.VarChar(50), createdByUserId)
        .input('LastModifiedDate', sql.DateTime, lastModifiedDate)
        .input('LastModifiedByUserId', sql.VarChar(50), lastModifiedByUserId)
        .execute('SaveDomain')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.SaveDomain = SaveDomain;
function DeleteDomain(vcc, domain) {
    return new sql.Request(sqlConn)
        .input('VCC', sql.VarChar(50), vcc)
        .input('Domain', sql.VarChar(50), domain)
        .execute('DeleteDomain')
        .then(function (recordsets) {
        var recordset = (recordsets && recordsets.length > 0 ? recordsets[0] : null);
        var output = (recordset && recordset.length > 0 ? recordset[0] : null);
        return Promise.resolve(output && output.result === 1);
    }).catch(function (err) {
        var errorMsg = (err && err.procName ? 'Error al ejecutar \'' + err.procName + '\': ' : 'Error al ejecutar consulta: ') + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.DeleteDomain = DeleteDomain;
