"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SavePlatformVersion = exports.GetPlatformVersion = exports.RemovePlatformStatusData = exports.SavePlatformStatusData = exports.GetPlatformStatusData = exports.PlatformRedis = void 0;
var log4js = require("log4js");
var redis_config_1 = require("../../config/redis.config");
var logger = log4js.getLogger('Redis');
var redisClient;
function PlatformRedis(app) {
    redisClient = app.get('redisClient');
}
exports.PlatformRedis = PlatformRedis;
function GetPlatformStatusData(storage, key) {
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        var platformSatusData = null;
        try {
            if (result) {
                platformSatusData = (typeof result === 'string') ? JSON.parse(result) : result;
            }
        }
        catch (err) {
            return Promise.resolve(null);
        }
        return Promise.resolve(platformSatusData);
    }).catch(function (err) {
        logger.error('[PlaformRedis::GetPlatformStatusData] getting err value:', err, 'hash:', storage, 'key:', key);
        return Promise.resolve(null);
    });
}
exports.GetPlatformStatusData = GetPlatformStatusData;
function SavePlatformStatusData(storage, key, platformSatusData) {
    if (!key || !storage)
        return Promise.resolve(false);
    return redisClient.hsetAsync(storage, key, JSON.stringify(platformSatusData))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_PLATFORM_DATA_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        logger.error('[PlaformRedis::SavePlatformStatusData] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SavePlatformStatusData = SavePlatformStatusData;
function RemovePlatformStatusData(storage, key) {
    return redisClient.hdelAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(result);
    })
        .catch(function (err) {
        logger.error('[PlatformRedis::RemovePlatformStatusData] getting error:', err);
        return Promise.resolve(err);
    });
}
exports.RemovePlatformStatusData = RemovePlatformStatusData;
function GetPlatformVersion() {
    var storage = 'PlatformVersion';
    var key = 'PlatformVersion';
    return redisClient.hgetAsync(storage, key)
        .then(function (result) {
        return Promise.resolve(JSON.parse(result));
    }).catch(function (err) {
        var errorMsg = 'Error al ejecutar consulta: ' + (err ? typeof err === 'string' ? err : err.message || err.description || '' : '');
        logger.error(errorMsg);
        return Promise.reject(err);
    });
}
exports.GetPlatformVersion = GetPlatformVersion;
function SavePlatformVersion(platformVersionData) {
    var storage = 'PlatformVersion';
    var key = 'PlatformVersion';
    return redisClient.hsetAsync(storage, key, JSON.stringify(platformVersionData))
        .then(function (result) {
        return redisClient.expireAsync(storage, redis_config_1.REDIS_PLATFORM_DATA_PERSISTENCE);
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        logger.error('[PlaformRedis::SavePlatformVersion] getting error:', err, 'with key:', key);
        return Promise.resolve(false);
    });
}
exports.SavePlatformVersion = SavePlatformVersion;
