"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.I6UpdateImplementation = void 0;
var config_1 = require("../config/config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var log4js = require("log4js");
var interaction_service_1 = require("../services/interaction.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('Task');
var CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
var CURRENT_IMPLEMENTATION = 'I6UpdateImplementation';
var I6UpdateImplementation = (function () {
    function I6UpdateImplementation() {
    }
    I6UpdateImplementation.prototype.Execute = function (executionCode, rateRecursive) {
        var _this = this;
        var executeIndex = 1;
        var ratio = config_1.RETRY_I6_UPDATE;
        var executeResult = true;
        return redis_service_1.Redis.LRange('I6Update', 0, ratio - 1)
            .then(function (result) {
            var promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[I6UpdateImplementation::Execute] executionCode:', executionCode, 'ratio:', ratio, 'getting Redis.LRange:', result);
            while (ratio >= executeIndex) {
                promiseArray.push(_this.I6UpdateExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then(function (results) {
            if (results.filter(function (item) { return item === false; }).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('I6Update', 0, ratio - 1);
        })
            .then(function (result) {
            if (result && result.length && rateRecursive >= 1)
                return _this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        })
            .then(function (result) {
            var currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[I6UpdateImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    };
    I6UpdateImplementation.prototype.I6UpdateExecute = function (executionCode) {
        var _this = this;
        var storage = '';
        var currentQueueId = '';
        var vcc = '';
        var processId = '';
        var batchType = '';
        var interactionToUpdate = null;
        return redis_service_1.Redis.RPopLPush('I6Update', 'I6UpdateExecute')
            .then(function (queueId) {
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId);
                var item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    batchType = item[2] ? item[2] : null;
                    storage = vcc + ':' + processId + ':i6update';
                    logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessI6Update(storage, currentQueueId);
                }
                else {
                    logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then(function (result) {
            if (result) {
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1');
                interactionToUpdate = result;
                return _this.UpdateProcessI6InteractionToUpdate(interactionToUpdate, executionCode, currentQueueId);
            }
            else {
                if (currentQueueId)
                    logger.warn('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (result) {
                interactionToUpdate = result;
                return redis_service_1.Redis.SaveProcessI6Update(storage, currentQueueId, interactionToUpdate);
            }
            return null;
        })
            .then(function (result) {
            var promiseArray = [];
            if (result && interactionToUpdate && interactionToUpdate.status === 1) {
                logger.info('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else if (interactionToUpdate && interactionToUpdate.retry >= config_1.RETRY_I6_UPDATE) {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionToUpdate) {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('I6UpdateExecute', currentQueueId));
            }
            else {
                logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var promiseArray = [];
            if (interactionToUpdate
                && interactionToUpdate.status === 1
                && interactionToUpdate.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessI6Update(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyI6UpdateInteraction(executionCode, currentQueueId, interactionToUpdate));
            }
            else if (interactionToUpdate
                &&
                    interactionToUpdate.retry >= config_1.RETRY_I6_UPDATE) {
                interactionToUpdate.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessI6Update(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyI6UpdateInteraction(executionCode, currentQueueId, interactionToUpdate));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processId, processId, CURRENT_IMPLEMENTATION, interactionToUpdate.id, null, interactionToUpdate.errorFunction, interactionToUpdate.errorMessage, interactionToUpdate.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            return _this.AddProcessQueue(vcc, processId, storage);
        })
            .then(function (_) {
            return Promise.resolve(true);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[I6UpdateImplementation::I6UpdateExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    };
    I6UpdateImplementation.prototype.UpdateProcessI6InteractionToUpdate = function (interactionToUpdate, executionCode, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            if (interactionToUpdate && interactionToUpdate.needI6Update) {
                return interaction_service_1.UpdateI6Interaction(interactionToUpdate);
            }
            else {
                return Promise.resolve(null);
            }
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_INTERACTION_I6_UPDATE_FAILED';
            if (interactionToUpdate) {
                interactionToUpdate.needI6Update = 0;
                interactionToUpdate['status'] = 1;
                interactionToUpdate['statusDescription'] = 'ready';
                interactionToUpdate['retry'] = 0;
                interactionToUpdate['errorMessage'] = null;
            }
            else {
                logger.error('[I6UpdateImplementation::UpdateProcessI6InteractionToUpdate] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interactionToUpdate['status'] = -1;
                interactionToUpdate['retry'] = interactionToUpdate['retry'] ? interactionToUpdate['retry'] + 1 : 1;
                interactionToUpdate['errorFunction'] = 'UpdateProcessI6InteractionToUpdate';
                interactionToUpdate['errorMessage'] = result;
                interactionToUpdate['errorDetail'] = result;
            }
            return Promise.resolve(interactionToUpdate);
        })
            .catch(function (err) {
            var errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[I6UpdateImplementation::UpdateProcessI6InteractionToUpdate] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interactionToUpdate['status'] = -1;
            interactionToUpdate['retry'] = interactionToUpdate['retry'] ? interactionToUpdate['retry'] + 1 : 1;
            interactionToUpdate['errorFunction'] = 'UpdateProcessI6InteractionToUpdate';
            interactionToUpdate['errorMessage'] = errorMessage;
            interactionToUpdate['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interactionToUpdate);
        });
        ;
    };
    I6UpdateImplementation.prototype.AddProcessQueue = function (vcc, processId, storage) {
        var readyToCreateProcessQueue = false;
        var processQueue = vcc + ":" + processId;
        return promises_shared_1.DummyPromise()
            .then(function (_) {
            return redis_service_1.Redis.ExistsProcessI6Update(storage);
        })
            .then(function (result) {
            if (!result) {
                readyToCreateProcessQueue = true;
                return redis_service_1.Redis.DeleteQueue('Process', processQueue);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (_) {
            if (readyToCreateProcessQueue) {
                logger.error('[I6UpdateImplementation::AddProcessQueue] Creating process queue');
                return redis_service_1.Redis.CreateQueueL('Process', processQueue);
            }
            else {
                return Promise.resolve(true);
            }
        });
    };
    return I6UpdateImplementation;
}());
exports.I6UpdateImplementation = I6UpdateImplementation;
