"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityService = void 0;
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var logger = log4js.getLogger('ServerScripts');
function EntityService(app) {
    var cognitiveService;
    app.get('/api/entity/get_entities/:app_id/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EntityService::get_entities] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[EntityService::get_entities] getting vcc:', vcc, 'azure:', azure);
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var appId = req.params.app_id || '';
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetEntitiesList(azure.subscriptionRegion, azure.subscriptionKey, appId);
        })
            .then(function (result) {
            logger.info('[EntityService::get_entities] getting', (result ? result.lenght : 0), 'items from cognitiveService.GetEntities');
            var idArray = result.map(function (a) { return a.id; });
            logger.info('[EntityService::get_entities] getting  stringId:', idArray.join('\',\''));
            return mssql_service_1.MSSql.GetEntities(vcc, appId, search_criteria, search_expression, idArray.join('\',\''));
        })
            .then(function (entities) {
            logger.info('[EntityService::get_entities] getting intents:', (entities ? entities.length : 0), 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: entities });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::get_entities] getting error:', err, 'of MSSql.GetIntents');
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/entity/get_entity/:customfield_id', function (req, res, next) {
    });
    app.post('/api/entity/create_entity', function (req, res, next) {
        logger.info('[EntityService::create_entity] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var azure = res.locals.azure;
        var entityType = 'SIMPLE';
        var newEntity = null;
        switch (entityType) {
            case 'SIMPLE':
                newEntity = newEntity;
                newEntity = (req.body || {});
                break;
        }
        newEntity.type = entityType;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!newEntity.name)
                throw 'SERVER_ERROR_ENTITY_NAME_NOT_FOUND';
            return mssql_service_1.MSSql.GetEntity(vcc, newEntity.appId, null, newEntity.name);
        })
            .then(function (result) {
            if (result)
                throw 'SERVER_ERROR_ENTITY_ALREADY_EXISTS';
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            var promise = null;
            switch (entityType) {
                case 'SIMPLE':
                    promise = cognitiveService.AddSimpleEntity(azure.subscriptionRegion, azure.subscriptionKey, newEntity.appId, newEntity);
                    break;
                default:
                    promise = Promise.resolve(null);
                    break;
            }
            return promise;
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_ENTITY_CREATE_FALED';
            newEntity.id = result;
            return mssql_service_1.MSSql.SaveEntity(vcc, newEntity.appId, newEntity.id, newEntity.name, newEntity.type, userId, null);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_ENTITY_SAVE_FALED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EntityService::create_entity] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/entity/delete_entity', function (req, res, next) {
        logger.info('[EntityService::delete_entity] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var entity = (req.body.entity || {});
        var appId = entity.appId;
        logger.info('[EntityService::delete_entity] getting entity:', entity.id);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            if (!entity)
                throw 'SERVER_ERROR_ENTITY_NOT_FOUND';
            return cognitiveService.DeleteAppEntity(azure.subscriptionRegion, azure.subscriptionKey, entity.appId, entity.id);
        })
            .then(function (result) {
            logger.info('[EntityService::delete_entity] getting result:', result, 'after of cognitiveService.DeleteAppEntity');
            return mssql_service_1.MSSql.DeleteEntity(vcc, appId, entity.id);
        })
            .then(function (result) {
            logger.info('[EntityService::delete_entity] getting result:', result, 'after of MSSql.DeleteEntity');
            if (!result)
                throw 'SERVER_ERROR_DELELE_ENTITY_FROM_DATABASE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'ERROR_EXECUTING_REQUEST');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::delete_entity] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/entity/get_nlus_entities', function (req, res, next) {
        logger.info('[EntityService::get_nlus_entities] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[EntityService::get_nlus_entities] getting vcc:', vcc, 'azure:', azure);
        var appIds = req.body.appIds || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            var promiseArray = [];
            var timeOut = 0;
            appIds.map(function (appId, index) {
                if (cognitiveService.GetCognitiveEngineName() === 'LUIS') {
                    timeOut = Math.floor(index / 5) * 1500;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve(cognitiveService.GetEntitiesList(azure.subscriptionRegion, azure.subscriptionKey, appId));
                        }, timeOut);
                    }));
                }
                else {
                    timeOut = index * 1000;
                    promiseArray.push(new Promise(function (resolve, reject) {
                        setTimeout(function () {
                            return resolve(cognitiveService.GetEntitiesList(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, appId));
                        }, timeOut);
                    }));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var modelEntities = [];
            appIds.map(function (appId, i) {
                modelEntities.push({
                    appId: appId,
                    entities: result[i] || []
                });
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: modelEntities });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[EntityService::get_nlus_entities] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    function CreateSimpleEntity(vcc, entity) {
        return Promise.resolve()
            .then(function (result) {
            return cognitiveService.GetIntentsList(null, null, entity.appId);
        })
            .then(function () {
            return mssql_service_1.MSSql.GetIntent(vcc, name);
        }).catch(function (err) {
            var errMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al ejecutar GetIntent: ' + errMsg);
        });
    }
}
exports.EntityService = EntityService;
