"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BillingImplementation = void 0;
var moment = require("moment");
var log4js = require("log4js");
var sy = require("systeminformation");
var promises_shared_1 = require("../shared/promises.shared");
var md5 = require("md5");
var mssql_service_1 = require("../db/mssql.service");
var account_model_1 = require("../models/account.model");
var redis_service_1 = require("../db/redis.service");
var platform_model_1 = require("../models/platform.model");
var platform_service_1 = require("../services/platform.service");
var icModule = require('intermediate-component-sa');
var logger = log4js.getLogger('Task');
var BillingImplementation = (function () {
    function BillingImplementation() {
    }
    BillingImplementation.prototype.SavePlatformVersion = function () {
        var genericResult = {};
        var version = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingBatchImplementation::SavePlatformVersion] Getting result after GetVersion');
            if (result && result.installationId) {
                version = result;
                var body = {
                    buildVersion: version.buildVersion
                };
                return icModule.InconcertRequest(version.installationId, 'IC_PARAM_URL_PLATFORM_VERSION_UPDATE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (result) {
                if (result.status === true) {
                    genericResult.status = true;
                    genericResult.data = result.data;
                    return mssql_service_1.MSSql.SaveVersionRegistration(version.installationId, version.buildVersion);
                }
                else {
                    genericResult.status = false;
                    genericResult.description = 'ERROR_FAILED_SEARCH';
                    return Promise.resolve(false);
                }
            }
            else {
                genericResult.status = false;
                genericResult.description = 'ERROR_FAILED_SEARCH';
                return Promise.resolve(false);
            }
        })
            .then(function (result) {
            return Promise.resolve(genericResult);
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::SavePlatformVersion] Getting error:', err);
            return Promise.reject(err);
        });
    };
    BillingImplementation.prototype.AddInsertTranscriptionDetailToMainFile = function (interaction, batchType, batchTypeId) {
        var entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        var line = '';
        line += 'NEW';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        line += '::';
        line += interaction.interactionType;
        line += '::';
        line += moment(interaction.startDate).utc().format('YYYY-MM-DD HH:mm:ss');
        line += '::';
        line += moment(interaction.endDate).utc().format('YYYY-MM-DD HH:mm:ss');
        line += '::';
        line += interaction.durationTime;
        line += '::';
        line += interaction.durationLinguisticTime;
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    };
    BillingImplementation.prototype.AddUpdateTranscriptionDetailToMainFile = function (interaction, target, batchType, batchTypeId) {
        var entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        var line = '';
        line += 'UPDATE';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        switch (target) {
            case 'ValidDurationTime':
                line += '::';
                line += 'ValidDurationTime';
                line += '::';
                line += interaction.durationLinguisticTime;
                break;
            case 'SpeechmaticsApi':
                line += '::';
                line += 'SpeechmaticsApi';
                line += '::';
                line += interaction.speechmaticsApi;
                break;
            case 'IdathaApi':
                line += '::';
                line += 'IdathaApi';
                line += '::';
                line += interaction.idathaApi;
                break;
            case 'ValidChar':
                line += '::';
                line += 'ValidChar';
                line += '::';
                line += interaction.validChar;
                break;
            case 'ValidWords':
                line += '::';
                line += 'ValidWords';
                line += '::';
                line += interaction.validWords;
                break;
            case 'TotalPhrases':
                line += '::';
                line += 'TotalPhrases';
                line += '::';
                line += interaction.totalPhrases;
                break;
            case 'CharsPerPhrase':
                line += '::';
                line += 'CharsPerPhrase';
                line += '::';
                line += interaction.charsPerPhrase;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    };
    BillingImplementation.prototype.AddCloseTranscriptionDetailToMainFile = function (interaction, batchType, batchTypeId, resultCode, resultDescription) {
        var entityId = '';
        if (interaction.interactionType === 'CALL') {
            entityId = interaction.id + '_' + interaction.section + '_' + interaction.part;
        }
        else {
            entityId = interaction.interactionType + '_' + interaction.campaign + '_' + interaction.folder + '_' + interaction.id;
        }
        var line = '';
        line += 'CLOSE';
        line += '::';
        line += interaction.vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'TRANSCRIPTION';
        line += '::';
        line += interaction.processId;
        line += '::';
        line += entityId;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    };
    BillingImplementation.prototype.AddInsertSemanticAnalysisDetailToMainFile = function (vcc, batchType, batchTypeId, processId, queryId) {
        var line = '';
        line += 'NEW';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        logger.info('[BillingImplementation::AddInsertSemanticAnalysisDetailToMainFile] Adding new Semantic Analysis to MainFile');
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    };
    BillingImplementation.prototype.AddUpdateSemanticAnalysisDetailToMainFile = function (vcc, processId, queryId, batchType, batchTypeId, updateTarget, updateValue) {
        var line = '';
        line += 'UPDATE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        switch (updateTarget) {
            case 'TotalValidRequest':
                line += '::';
                line += 'TotalValidRequest';
                line += '::';
                line += updateValue;
                break;
            case 'TotalInvalidRequest':
                line += '::';
                line += 'TotalInvalidRequest';
                line += '::';
                line += updateValue;
                break;
            case 'TotalResults':
                line += '::';
                line += 'TotalResults';
                line += '::';
                line += updateValue;
                break;
            case 'TotalPages':
                line += '::';
                line += 'TotalPages';
                line += '::';
                line += updateValue;
                break;
            case 'TotalItemsPerPage':
                line += '::';
                line += 'TotalItemsPerPage';
                line += '::';
                line += updateValue;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    };
    BillingImplementation.prototype.AddCloseSemanticAnalysisDetailToMainFile = function (vcc, processId, queryId, batchType, batchTypeId, resultCode, resultDescription) {
        var line = '';
        line += 'CLOSE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'SEMANTIC_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += queryId;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    };
    BillingImplementation.prototype.AddInsertCognitiveAnalysisDetailToMainFile = function (vcc, batchType, batchTypeId, processId, nluModel) {
        var line = '';
        line += 'NEW';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        logger.info('[BillingImplementation::AddInsertCognitiveAnalysisDetailToMainFile] Adding new Cognitive Analysis to MainFile');
        icModule.InconcertAddDataToGeneralFile(line, 'new');
    };
    BillingImplementation.prototype.AddUpdateCognitiveAnalysisDetailToMainFile = function (vcc, processId, nluModel, updateTarget, updateValue, batchType, batchTypeId) {
        var line = '';
        line += 'UPDATE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        switch (updateTarget) {
            case 'TotalValidRequest':
                line += '::';
                line += 'TotalValidRequest';
                line += '::';
                line += 1;
                break;
            case 'TotalInvalidRequest':
                line += '::';
                line += 'TotalInvalidRequest';
                line += '::';
                line += 1;
                break;
            case 'IsValid':
                line += '::';
                line += 'IsValid';
                line += '::';
                line += updateValue ? 1 : 0;
                break;
            case 'ApiId':
                line += '::';
                line += 'ApiId';
                line += '::';
                line += updateValue;
                break;
            case 'IsDemo':
                line += '::';
                line += 'IsDemo';
                line += '::';
                line += updateValue ? 1 : 0;
                break;
            default:
                line += '::';
                line += updateTarget;
                line += '::';
                line += updateValue ? 1 : 0;
                break;
        }
        icModule.InconcertAddDataToGeneralFile(line, 'update');
    };
    BillingImplementation.prototype.AddCloseCognitiveAnalysisDetailToMainFile = function (vcc, processId, nluModel, batchType, batchTypeId, resultCode, resultDescription) {
        var line = '';
        line += 'CLOSE';
        line += '::';
        line += vcc;
        line += '::';
        line += batchType;
        line += '::';
        line += batchTypeId;
        line += '::';
        line += 'COGNITIVE_ANALYSIS';
        line += '::';
        line += processId;
        line += '::';
        line += nluModel;
        line += '::';
        line += resultCode;
        line += '::';
        line += resultDescription;
        icModule.InconcertAddDataToGeneralFile(line, resultCode ? 'success' : 'failed');
    };
    BillingImplementation.prototype.SplitGeneralFileToSegmentedFiles = function () {
        icModule.InconcertSplitGeneralFileData();
    };
    BillingImplementation.prototype.SegmentedFilesUpload = function (installationId) {
        return icModule.InconcertSegmentedFileUpload(installationId)
            .then(function (result) {
            logger.info('[BillingImplementation::SegmentedFilesUpload] Getting result:', result);
            return Promise.resolve(true);
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::SegmentedFilesUpload] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetAccountsSubAccounts = function (vcc, updated) {
        var _this = this;
        if (updated === void 0) { updated = false; }
        var storage = '';
        var key = '';
        var installationId = '';
        var isForced = updated;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            if (isForced === false) {
                return redis_service_1.Redis.GetPlatformStatusData(storage, key);
            }
            else {
                return redis_service_1.Redis.RemovePlatformStatusData(storage, key);
            }
        })
            .then(function (result) {
            var accountList = null;
            if (result && result.accountList && result.subAccountList) {
                accountList = new account_model_1.AccountList(result.accountList, result.subAccountList);
            }
            return Promise.resolve(accountList);
        })
            .then(function (result) {
            if (result === null) {
                return _this.GetPlatformStatus(vcc);
            }
            return result;
        })
            .then(function (result) {
            if (result && isForced) {
                return redis_service_1.Redis.SavePlatformStatusData(storage, key, result);
            }
            return Promise.resolve(result);
        })
            .then(function (result) {
            if (typeof result == 'boolean') {
                if (result) {
                    return _this.GetAccountsSubAccounts(vcc);
                }
                else {
                    throw 'FAILED_SAVE_IN_REDI_PLATFORM_STATUS';
                }
            }
            else {
                return Promise.resolve(result);
            }
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::GetAccountsSubAccounts] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetPlatformAccounts = function (vcc) {
        var storage = '';
        var key = '';
        var installationId = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            return redis_service_1.Redis.GetPlatformStatusData(storage, key);
        }).then(function (result) {
            logger.info('[BillingImplementation::GetPlatformAccounts] Getting result:', result, 'after external api/account/get_platform_accounts request');
            var platformAccount = [];
            if (result && result.platformAccounts) {
                platformAccount = result.platformAccounts;
            }
            return Promise.resolve(platformAccount);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformAccounts] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.ActivatePlatform = function (account, token, user) {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[BillingImplementation::ActivatePlatform] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var platformHash = '';
            if (systemInformationData) {
                platformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                account: account,
                token: token,
                user: user,
                data: result,
                platformHash: platformHash
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_ACTIVATE', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::ActivatePlatform] Getting result:', result, 'from external server');
            return Promise.resolve(result);
        }).catch(function (err) {
            logger.error('[BillingImplementation::ActivatePlatform] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetPlatformStatus = function (vcc) {
        var _this = this;
        var storage = '';
        var key = '';
        var installationId = '';
        var mustSaveRedis = false;
        var platformStatusData = null;
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            logger.info('[BillingImplementation::GetPlatformStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            systemInformationData = result;
            storage = vcc;
            storage += ':billing:';
            storage += installationId;
            key = 'platformStatus';
            return redis_service_1.Redis.GetPlatformStatusData(storage, key);
        })
            .then(function (result) {
            if (result
                && result.platformStatus && result.platformStatus.macAddress && result.platformStatus.macAddress === systemInformationData.macAddress
                && result.accountList && result.accountList.length
                && result.subAccountList && result.subAccountList.length) {
                platformStatusData = new platform_model_1.PlatformSatusData();
                platformStatusData = result;
                return platformStatusData;
            }
            else {
                mustSaveRedis = true;
                var body = systemInformationData;
                return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_GET_STATUS', body);
            }
        })
            .then(function (result) {
            if (result && result.status === false)
                throw result.error;
            if (result && result.status && result.data && result.data.platformStatus && result.data.accountList && result.data.subAccountList) {
                platformStatusData = result.data;
            }
            if (mustSaveRedis && platformStatusData && platformStatusData.platformStatus && platformStatusData.accountList && platformStatusData.subAccountList) {
                return redis_service_1.Redis.SavePlatformStatusData(storage, key, platformStatusData);
            }
            return true;
        })
            .then(function (result) {
            return Promise.resolve(platformStatusData);
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.reject(err);
        });
    };
    BillingImplementation.prototype.GetPlatformAvailableLicenses = function () {
        var _this = this;
        var systemInformationData = null;
        var installationId;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[BillingImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                macAddress: systemInformationData.macAddress,
                baseboardSerialNumber: systemInformationData.baseboardSerialNumber
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_AVAILABLE_LICENSE', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::GetPlatformAvailableLicenses] Getting result:', result, 'from external server');
            var products = [];
            if (result && result.data) {
                products = result.data;
            }
            return Promise.resolve(products);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformAvailableLicenses] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetSystemInformation = function (guid) {
        var systemInformationData = {
            installationId: guid,
            macAddress: '',
            publicIp: ''
        };
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            logger.info('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting result serial:', result.serial);
            var sys = sy.networkInterfaces();
            return sys;
        }).then(function (result) {
            if (result && result.length) {
                logger.info('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting result[0].mac:', result[0].mac);
                var myNetworkInterface = null;
                myNetworkInterface = result.filter(function (r) { return r.operstate === 'up'; })[0];
                systemInformationData.macAddress = myNetworkInterface.mac;
                systemInformationData.publicIp = myNetworkInterface.ip4;
            }
            return Promise.resolve(systemInformationData);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetSystemInformation] guid:', guid, 'getting error:', err);
            return Promise.resolve(false);
        });
    };
    BillingImplementation.prototype.GenerateLicenseFileData = function (guid) {
        var _this = this;
        var installationId = '';
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
            }
            else {
                throw 'NO_INSTALLATION_ID_PARAMETER';
            }
            return _this.GetSystemInformation(installationId);
        }).then(function (result) {
            logger.info('[BillingImplementation::GenerateLicenseFileData] guid:', guid, 'getting result:', result, 'after this.GetSystemInformation');
            return Promise.resolve(result);
        }).then(function (result) {
            var licenseFileContent = '';
            if (result) {
                licenseFileContent = _this.GenerateMD5Content(result);
                logger.info('[BillingImplementation::GenerateLicenseFileData] getting licenseFileContent:', licenseFileContent);
            }
            return Promise.resolve(licenseFileContent);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GenerateLicenseFileData] guid:', guid, 'getting error:', err);
            return Promise.resolve('ERROR');
        });
    };
    BillingImplementation.prototype.GenerateMD5Content = function (data) {
        var result = '';
        if (data) {
            result += data.installationId;
            result += data.macAddress;
            result = md5(result);
        }
        return result;
    };
    BillingImplementation.prototype.CreateVCC = function (vcc, productVCC) {
        var _this = this;
        var installationId = '';
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[BillingImplementation::GetPlatformStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var products = [];
            if (productVCC.products && productVCC.products.length) {
                products = productVCC.products.map(function (x) {
                    return x.product;
                });
            }
            var account = {};
            account.vcc = vcc;
            account.accountId = productVCC.accountId;
            account.subAccountId = productVCC.subAccountId;
            account.asignableBalance = productVCC.asignableBalance;
            account.installationId = installationId;
            account.transcriptionRatePerHour = productVCC.transcriptionRatePerHour;
            account.transcriptionRatePerSecond = productVCC.transcriptionRatePerSecond;
            account.semanticAnalysisRatePerTransaction = productVCC.semanticAnalysisRatePerTransaction;
            account.cognitiveAnalysisRatePerTransaction = productVCC.cognitiveAnalysisRatePerTransaction;
            var body = {
                vcc: vcc,
                platformHash: plaformHash,
                account: productVCC.accountId,
                subAccount: productVCC.subAccountId,
                products: products,
                accountVcc: account
            };
            logger.info('[BillingImplementation::CreateVCC] Getting body:', JSON.stringify(body), 'before sendig request to api/vcc/create_vcc');
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_CREATE', body);
        })
            .then(function (result) {
            var rpta = {
                status: result.status,
                description: result.description ? result.description : null,
                error: result.error ? result.error : null
            };
            return Promise.resolve(rpta);
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.resolve(false);
        });
    };
    BillingImplementation.prototype.UpdateVCC = function (vcc, productVCC) {
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        })
            .then(function (result) {
            if (result && result.installationId) {
                var products = [];
                if (productVCC.products && productVCC.products.length) {
                    products = productVCC.products.map(function (x) {
                        return x.product;
                    });
                }
                var body = {
                    vcc: vcc,
                    products: products
                };
                logger.info('[BillingImplementation::UpdateVCC] Getting body:', JSON.stringify(body), 'before sendig request to api/vcc/UpdateVCC');
                return icModule.InconcertRequest(result.installationId, 'IC_PARAM_URL_VCC_UPDATE', body);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            var rpta;
            if (result) {
                rpta = result.status;
            }
            else {
                rpta = false;
            }
            return Promise.resolve(rpta);
        })
            .catch(function (err) {
            logger.error('[BillingImplementation::UpdateVCC] Getting error:', err);
            return Promise.resolve(false);
        });
    };
    BillingImplementation.prototype.GetVCCStatusDetail = function (vcc) {
        var _this = this;
        var installationId = '';
        var vccStatusData = null;
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[BillingImplementation::GetVCCStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                vcc: vcc,
                platformHash: plaformHash,
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_STATUS', body);
        }).then(function (result) {
            if (result.data) {
                vccStatusData = result.data;
            }
            else {
                throw 'PLATFORM_IS_NOT_ACTIVATED';
            }
            return Promise.resolve(vccStatusData);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.resolve(err);
        });
    };
    BillingImplementation.prototype.GetVCCStatus = function (vcc) {
        var _this = this;
        var installationId = '';
        var vccStatusData = null;
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[BillingImplementation::GetVCCStatus] Getting result:', result, 'after GetVersion');
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                vcc: vcc,
                platformHash: plaformHash,
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_STATUS', body);
        }).then(function (result) {
            if (result.data) {
                vccStatusData = result.data;
            }
            else {
                throw 'PLATFORM_IS_NOT_ACTIVATED';
            }
            return Promise.resolve(vccStatusData);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetPlatformStatus] Getting error:', err);
            return Promise.resolve(false);
        });
    };
    BillingImplementation.prototype.GetVCCRates = function (vcc, processId) {
        var _this = this;
        var installationId = '';
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: systemInformationData.installationId,
                vcc: vcc,
                processId: processId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_RATES', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::GetVCCRates] Getting result:', result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE', ' from external server');
            var vccRates = null;
            if (result && result.data) {
                vccRates = result.data;
            }
            return Promise.resolve(vccRates);
        }).catch(function (err) {
            logger.error('[BillingImplementation::GetVCCRates] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetVCCBalance = function (vcc) {
        var _this = this;
        var installationId = '';
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var body = {
                platformHash: plaformHash,
                installationId: installationId,
                vcc: vcc
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_GET_BALANCE', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::GetVCCBalance] Getting result:', result.length, 'from external server');
            var vccBalance = null;
            if (result && result.data) {
                vccBalance = result.data;
            }
            return Promise.resolve(vccBalance);
        }).catch(function (err) {
            console.log(err);
            logger.error('[BillingImplementation::GetVCCBalance] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetInstallationData = function (installationId) {
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            var body = {
                installationId: installationId
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_PLATFORM_INSTALLATION_DATA', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::GetVCCBalance] Getting result:', result.length, 'from external server');
            var response = [];
            if (result.status && result.data) {
                response = result.data;
            }
            return Promise.resolve(response);
        }).catch(function (err) {
            console.log(err);
            logger.error('[BillingImplementation::GetVCCBalance] Getting error:', err);
            return Promise.resolve(null);
        });
    };
    BillingImplementation.prototype.GetExternalLibrary = function () {
        var icModule = null;
        icModule = require('intermediate-component-sa');
        return icModule;
    };
    BillingImplementation.prototype.SaveVCCBalance = function (vcc, account, subAccount, transcriptionOld, transcriptionNew, transcriptonDetail, cognitiveOld, cognitiveNew, cognitiveDetail, semanticOld, semanticNew, semanticDetail, freeOld, freeNew, freeDetail) {
        var _this = this;
        var installationId = '';
        var systemInformationData = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            if (result && result.installationId) {
                installationId = result.installationId;
                return _this.GetSystemInformation(result.installationId);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            systemInformationData = result;
            var plaformHash = '';
            if (systemInformationData) {
                plaformHash = _this.GenerateMD5Content(systemInformationData);
            }
            var data = [];
            data.push({ target: 'transcription', total: transcriptionNew, balance: transcriptionOld, operation: transcriptonDetail });
            data.push({ target: 'semantic_analysis', total: semanticNew, balance: semanticOld, operation: semanticDetail });
            data.push({ target: 'cognitive_analysis', total: cognitiveNew, balance: cognitiveOld, operation: cognitiveDetail });
            data.push({ target: 'free', total: freeNew, balance: freeOld, operation: freeDetail });
            var body = {
                platformHash: plaformHash,
                installationId: installationId,
                vcc: vcc,
                account: account,
                subAccount: subAccount,
                data: data
            };
            return icModule.InconcertRequest(installationId, 'IC_PARAM_URL_VCC_BALANCE_ALLOCATION', body);
        }).then(function (result) {
            logger.info('[BillingImplementation::SaveVCCBalance] Getting result:', result, 'from external server');
            return Promise.resolve(result.status);
        }).catch(function (err) {
            logger.error('[BillingImplementation::SaveVCCBalance] Getting error:', err, 'vcc:', vcc);
            return Promise.resolve(null);
        });
    };
    return BillingImplementation;
}());
exports.BillingImplementation = BillingImplementation;
