"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityMigrator = void 0;
var redis_service_1 = require("../../db/redis.service");
var log4js = require("log4js");
var mssql_service_1 = require("../../db/mssql.service");
var promises_shared_1 = require("../../shared/promises.shared");
var helper_1 = require("./helper");
var enum_1 = require("./enum");
var arrays_shared_1 = require("../../shared/arrays.shared");
var logger = log4js.getLogger('Task');
var EntityMigrator = (function () {
    function EntityMigrator() {
    }
    EntityMigrator.prototype.MigrateEntities = function (vcc, data, start, end) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Migrating entities for ', vcc, ' into ', data.newEngine, ' from ', start, ' and ', end);
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.LRange(enum_1.ROOT_KEY + ":" + vcc + ":Entities:Keys", start, end - 1);
        }).then(function (keys) {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Found keys for entities ', keys);
            if (keys && keys.length > 0) {
                return redis_service_1.Redis.GetCognitiveEngineMigrationModel(enum_1.ROOT_KEY + ":" + vcc + ":Entities", arrays_shared_1.GetUniqueFromArray(keys));
            }
            return [];
        }).then(function (entities) {
            logger.info('[CognitiveEngineMigrateImplementation::MigrateEntities] Found  entities ', entities);
            if (entities) {
                var entityList = [];
                for (var _i = 0, entities_1 = entities; _i < entities_1.length; _i++) {
                    var e = entities_1[_i];
                    var entity = JSON.parse(e);
                    if (entity) {
                        entityList.push(entity);
                    }
                }
                if (entityList.length > 0) {
                    return _this.ProcessEntities(vcc, data, entityList);
                }
            }
            return false;
        }).then(function (_) {
            if (end < data.totalEntity) {
                return _this.MigrateEntities(vcc, data, start + enum_1.ITEMS_PER_PAGE, end + enum_1.ITEMS_PER_PAGE);
            }
            return true;
        });
    };
    EntityMigrator.prototype.ProcessEntities = function (vcc, data, entities) {
        var _this = this;
        logger.info('[CognitiveEngineMigrateImplementation::ProcessEntities] Starting process entities:', entities.length);
        return promises_shared_1.DummyPromise().then(function (_) {
            var promises = [];
            for (var _i = 0, entities_2 = entities; _i < entities_2.length; _i++) {
                var entity = entities_2[_i];
                switch (entity.status) {
                    case 0:
                        promises.push(_this.AddEntity(vcc, data, entity));
                        break;
                    default: break;
                }
            }
            return Promise.all(promises);
        }).then(function (results) {
            logger.info('[CognitiveEngineMigrateImplementation::ProcessEntities] Results for process entities:', results);
            if (results.length > 0) {
                var totalCreated = results.filter(function (r) { return r == true; }).length;
                var totalError = results.filter(function (r) { return !r; }).length;
                return redis_service_1.Redis.UpdateCognitiveEngineMigrationData(vcc, Object.assign(data, {
                    totalCreatedEntity: totalCreated + data.totalCreatedEntity,
                    totalErrorEntity: totalError + data.totalErrorEntity
                }));
            }
            return false;
        });
    };
    EntityMigrator.prototype.AddEntity = function (vcc, data, entity) {
        logger.info('[CognitiveEngineMigrateImplementation::AddEntity] Adding entity ', entity.name, ' into ', data.newEngine);
        var AppId = null;
        return promises_shared_1.DummyPromise().then(function (_) {
            return redis_service_1.Redis.GetCognitiveEngineMigrationNewId(enum_1.ROOT_KEY + ":" + vcc + ":AppIds", entity.appId);
        }).
            then(function (appId) {
            if (appId) {
                AppId = appId;
                return helper_1.GetServiceInstance(data.newEngine);
            }
            logger.error('[CognitiveEngineMigrateImplementation::AddEntity] Error on get new app id from redis for ', entity.key);
            return null;
        })
            .then(function (service) {
            if (service && AppId) {
                return service.AddSimpleEntity(data.subscriptionRegion, data.subscriptionKey, AppId, entity);
            }
            return null;
        })
            .then(function (newEntityId) {
            if (newEntityId) {
                var promises = [];
                promises.push(Promise.resolve(newEntityId));
                promises.push(mssql_service_1.MSSql.UpdateEntityId(vcc, AppId, entity.id, newEntityId));
                return Promise.all(promises);
            }
            logger.error("[CognitiveEngineMigrateImplementation::AddEntity] Error on add entity into " + data.newEngine + ":", newEntityId);
            return [null];
        })
            .then(function (result) {
            var newEntityId = result[0];
            if (newEntityId) {
                return mssql_service_1.MSSql.SaveCognitiveEngineMigrateStatus(vcc, 'entity', newEntityId, 1, enum_1.AVAILABLE);
            }
            return false;
        })
            .then(function (result) {
            logger.info('[CognitiveEngineMigrateImplementation::AddEntity] Updated new entity id into db:', result);
            if (result) {
                return redis_service_1.Redis.DeleteCognitiveEngineMigrationModel(enum_1.ROOT_KEY + ":" + vcc + ":Entities", entity.key);
            }
            return false;
        }).catch(function (err) {
            logger.error("[CognitiveEngineMigrateImplementation::AddEntity] Unknown error on add entity into " + data.newEngine, err);
            return false;
        });
    };
    return EntityMigrator;
}());
exports.EntityMigrator = EntityMigrator;
