"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetAbilityQueries = exports.AbilityService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var logger = log4js.getLogger('ServerScripts');
function AbilityService(app) {
    app.get('/api/ability/get_ability_detail/:abilityId', function (req, res) {
        logger.info('[AbilityService::get_ability_detail] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var id = req.params.abilityId || '';
        promises_shared_1.DummyPromise().then(function (_) {
            return mssql_service_1.MSSql.GetAbilityDetail(vcc, id);
        }).
            then(function (items) {
            res.json({ status: items ? true : false, data: items });
        }).
            catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_abilitys/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[AbilityService::get_abilitys] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilitys(instance, search_criteria, search_expression, disabled);
        }).then(function (abilitys) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: abilitys });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::get_abilitys] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/create_ability', function (req, res, next) {
        logger.info('[AbilityService::create_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newAbility = (req.body || {});
        newAbility.createdDate = nowUtc;
        newAbility.createdByUserId = userId;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        newAbility.title = newAbility.title.trim();
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetAbilityTitle(instance, newAbility.title);
        }).then(function (result) {
            if (result)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            newAbility.id = utilities_1.guid();
            return SaveAbility(instance, newAbility);
        })
            .then(function (result) {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            if (newAbility.queries && newAbility.queries.length > 0) {
                return SaveAbilityQueries(instance, newAbility, userId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_ABILITY_QUERY_SAVE';
            return SaveScore(instance, newAbility);
        }).then(function (result) {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_ON_SAVE';
            return Promise.resolve(newAbility.id);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'Abilidad save failed'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::create_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/ability/get_ability/:ability_id', function (req, res, next) {
        logger.info('[AbilityService::get_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var ability = {};
        var scores = [];
        var abilityId = req.params.ability_id;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.GetAbility(instance, abilityId);
        })
            .then(function (result) {
            if (!abilityId)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            ability = result || ability;
            return GetAbilityQueries(instance, abilityId, true);
        })
            .then(function (result) {
            ability.queries = result;
            return mssql_service_1.MSSql.GetScores(instance, abilityId);
        }).then(function (resultScores) {
            if (res.headersSent)
                return;
            scores = resultScores || scores;
            ability.scores = scores;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: ability });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/save_ability', function (req, res, next) {
        logger.info('[AbilityService::save_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var userId = req.app.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newAbility = (req.body || {});
        newAbility.createdDate = null;
        newAbility.createdByUserId = null;
        newAbility.lastModifiedDate = nowUtc;
        newAbility.lastModifiedByUserId = userId;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAbilityTitle(instance, newAbility.title);
        }).then(function (result) {
            var abilityDesc = {};
            abilityDesc = result || abilityDesc;
            if (result && abilityDesc.id !== newAbility.id)
                throw 'ABILITY_SERVER_ERROR_EXISTS';
            return mssql_service_1.MSSql.SaveAbility(instance, newAbility.id, newAbility.title, newAbility.detail, newAbility.type, newAbility.binary, newAbility.notApply, newAbility.distribute, newAbility.acceptComments, newAbility.createdDate, newAbility.createdByUserId, newAbility.lastModifiedDate, newAbility.lastModifiedByUserId, newAbility.enabled, newAbility.analysisType, newAbility.language, newAbility.isNegative);
        })
            .then(function (result) {
            return mssql_service_1.MSSql.DeleteAbilityQueries(instance, newAbility.id);
        })
            .then(function (result) {
            if (newAbility.queries && newAbility.queries.length > 0) {
                return SaveAbilityQueries(instance, newAbility, userId);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(function (resultIdAbility) {
            return mssql_service_1.MSSql.DeleteScore(instance, newAbility.id);
        }).then(function (resultDeleteScore) {
            return SaveScore(instance, newAbility);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_ON_SAVE'), data: resultSave });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::save_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_ability', function (req, res, next) {
        logger.info('[AbilityService::delete_ability] Received ' + req.method + ' method on \'' + req.path + '\'');
        var instance = res.locals.instance;
        var ability = (req.body || {});
        var abilityId = ability.id;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!ability)
                throw 'ABILITY_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteScore(instance, abilityId);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteAbility(instance, abilityId);
        }).then(function (result) {
            if (!result)
                throw 'ABILITY_SERVER_ERROR_CANNOT_DELETE';
            return Promise.resolve(true);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_CANNOT_DELETE'), data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AbilityService::delete_ability] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/ability/delete_abilitys', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var instance = res.locals.instance;
        var abilitys = req.body.abilitys || [];
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!abilitys)
                throw 'ABILITY_SERVER_ERROR_MISING_VCC';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteScore(instance, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!abilitys)
                throw 'ABILITY_SERVER_ERROR_MISING_VCC';
            var promiseArray = [];
            abilitys.forEach(function (ability) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAbility(instance, ability.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeleteVccs) {
            var result;
            if (resultDeleteVccs.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Vccs eliminados correctamente');
            }
            return result;
        }).then(function (resultDeleteVccs) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteVccs, description: (resultDeleteVccs ? 'SERVER_SUCCESS_MESSAGE' : 'ABILITY_SERVER_ERROR_CANNOT_DELETE'), data: resultDeleteVccs });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.AbilityService = AbilityService;
function SaveAbility(instance, ability) {
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveAbility(instance, ability.id, ability.title, ability.detail, ability.type, ability.binary, ability.notApply, ability.distribute, ability.acceptComments, ability.createdDate, ability.createdByUserId, ability.lastModifiedDate, ability.lastModifiedByUserId, ability.enabled, ability.analysisType, ability.language, ability.isNegative));
    return Promise.all(promiseArray).then(function (results) {
        var falseResults = results.filter(function (res) { return res[0].result !== true && res[0].result !== 1; });
        if (falseResults.length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
function SaveAbilityQueries(vcc, ability, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        var promiseArray = [];
        if (ability && ability.queries && ability.queries.length > 0) {
            ability.queries.map(function (aq) {
                aq.id = utilities_1.guid();
                aq.abilityId = ability.id;
                promiseArray.push(SaveAbilityQuery(vcc, aq, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var saveResult = false;
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQueries] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQuery(vcc, abilityQuery, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (result) {
        return mssql_service_1.MSSql.SaveAbilityQuery(vcc, abilityQuery.id, abilityQuery.abilityId, abilityQuery.queryId, abilityQuery.queryContent, abilityQuery.acceptancePercentage, abilityQuery.order, userId, userId);
    })
        .then(function (result) {
        var promiseArray = [];
        if (abilityQuery.entities && abilityQuery.entities.length > 0) {
            abilityQuery.entities.map(function (e) {
                e.abilityQueryId = abilityQuery.id;
                promiseArray.push(SaveAbilityQueryEntity(vcc, e, userId));
            });
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        var saveResult = false;
        if (result.filter(function (r) { return (r !== true); }).length > 0) {
            saveResult = false;
        }
        else {
            saveResult = true;
        }
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::SaveAbilityQuery] Error: ' + errorMsg);
        return Promise.resolve(false);
    });
}
function SaveAbilityQueryEntity(vcc, abilityQueryEntity, userId) {
    return mssql_service_1.MSSql.SaveAbilityQueryEntity(vcc, abilityQueryEntity.abilityQueryId, abilityQueryEntity.entityId, abilityQueryEntity.entityName, abilityQueryEntity.acceptancePercentage, abilityQueryEntity.order, userId, userId);
}
function GetAbilityQueries(vcc, abilityId, getEntities, evaluationPlanId) {
    if (getEntities === void 0) { getEntities = false; }
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    var abilityQueries = [];
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetAbilityQueries(vcc, abilityId, evaluationPlanId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_ABILITY_QUERIES_NOT_FOUND';
        if (getEntities) {
            return GetAbilityQueryEntities(vcc, result, evaluationPlanId);
        }
        else {
            result.map(function (r) { return r.entities = []; });
            return Promise.resolve(result);
        }
    })
        .then(function (result) {
        abilityQueries = result;
        return Promise.resolve(abilityQueries);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[AbilityService::GetAbilityQueries] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
exports.GetAbilityQueries = GetAbilityQueries;
function GetAbilityQueryEntities(vcc, abilityQueries, evaluationPlanId) {
    if (evaluationPlanId === void 0) { evaluationPlanId = null; }
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        var promiseArray = [];
        abilityQueries.map(function (aq) {
            promiseArray.push(mssql_service_1.MSSql.GetAbilityQueryEntities(vcc, aq.id, evaluationPlanId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        result.map(function (r, i) {
            abilityQueries[i].entities = r;
        });
        return Promise.resolve(abilityQueries);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[Abilityervice::GetAbilityQueryEntities] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
function SaveScore(instance, ability) {
    var promiseArray = [];
    if (ability.scores) {
        ability.scores.forEach(function (score) {
            promiseArray.push(mssql_service_1.MSSql.SaveScore(instance, score.id, ability.id, score.description, score.value));
        });
    }
    return Promise.all(promiseArray).then(function (results) {
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
