"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateNewBatcScheduleExecutions = exports.SaveEvaluationPlanBatchSchedule = exports.SaveProcessBatchSchedule = exports.BatchScheduleService = void 0;
var log4js = require("log4js");
var moment = require("moment");
var mssql_service_1 = require("../db/mssql.service");
var schedule_model_1 = require("../models/schedule.model");
var promises_shared_1 = require("../shared/promises.shared");
var utilities_1 = require("../shared/utilities");
var batch_schedule_implementation_1 = require("../implementation/batch-schedule.implementation");
var logger = log4js.getLogger('ServerScripts');
function BatchScheduleService(app) {
    app.post('/api/schedule/get_schedule', function (req, res, next) {
        logger.info('[BatchScheduleService::get_schedule] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/get_schedules', function (req, res, next) {
        logger.info('[BatchScheduleService::get_schedules] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var searchCriteria = bodyData.searchCriteria;
        var searchExpression = bodyData.searchExpression;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchSchedulesSummary(vcc, searchCriteria, searchExpression);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedules] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/get_schedule_executions', function (req, res, next) {
        logger.info('[BatchScheduleService::get_schedule_executions] Received ' + req.method + ' methos on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByBatchType(vcc, batchScheduleId, null);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::get_schedule_executions] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/schedule/save_schedule_status', function (req, res, next) {
        logger.info('[BatchScheduleService::save_schedule_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var bodyData = (req.body || {});
        var batchScheduleId = bodyData.scheduleId;
        var newStatus = bodyData.status;
        var scheduleData = null;
        var saveResult = false;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            return mssql_service_1.MSSql.GetBatchScheduleById(vcc, batchScheduleId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_NOT_FOUND';
            scheduleData = result;
            switch (newStatus) {
                case 'PAUSE':
                    scheduleData.status = 'PAUSED';
                    break;
                case 'RESUME':
                    scheduleData.status = 'RUNNING';
                    break;
            }
            return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_BATCH_SCHEDULE_STATUS_UPDATE_FAILED';
            saveResult = result;
            var promiseArray = [];
            switch (newStatus) {
                case 'PAUSE':
                    promiseArray.push(DeleteBatchsScheduleExecutionsData(vcc, scheduleData));
                    break;
                case 'RESUME':
                    promiseArray.push(CreateBatchScheduleExecutionsData(vcc, scheduleData));
                    break;
            }
            return Promise.all(promiseArray);
        })
            .then(function (_) {
            if (res.headersSent)
                return;
            res.json({ status: saveResult, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'STUDY_SERVER_ERROR_ON_SAVE'), data: saveResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::save_schedule_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/schedule/delete_schedule', function (req, res, next) {
        logger.info('[BatchScheduleService::delete_schedule] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var batchSchedule = req.body.schedule;
        promises_shared_1.DummyPromise()
            .then(function (_) {
            if (!batchSchedule)
                throw 'SERVER_ERROR_MISSING_SCHEDULE_ID';
            return mssql_service_1.MSSql.DeleteBatchSchedule(vcc, batchSchedule.id);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SCHEDULE_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_SCHEDULE_DELETE_FAILED'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[BatchScheduleService::delete_schedule] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.BatchScheduleService = BatchScheduleService;
function SaveBatchSchedule(vcc, scheduleData, userId) {
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return mssql_service_1.MSSql.GetBatchScheduleById(vcc, scheduleData.id);
    })
        .then(function (result) {
        if (result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_ALREADY_EXIST';
        return mssql_service_1.MSSql.SaveBatchSchedule(vcc, scheduleData, userId);
    })
        .then(function (_) {
        return CreateBatchScheduleExecutionsData(vcc, scheduleData);
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::SaveBatchSchedule] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
function CreateBatchScheduleExecutionsData(vcc, scheduleData) {
    var executionCode = moment().toDate().toString();
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return batchScheduleImplementation.GenerateNewScheduleBatchExecution(executionCode, scheduleData.id);
    })
        .then(function (_) {
        return batchScheduleImplementation.GetNewScheduledBatchExecutions(executionCode, scheduleData.id);
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::CreateBatchScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::CreateBatchScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function DeleteBatchsScheduleExecutionsData(vcc, scheduleData) {
    var executionCode = moment().toDate().toString();
    var batchScheduleImplementation = new batch_schedule_implementation_1.BatchScheduleImplementation();
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return batchScheduleImplementation.RemoveScheduleBatchExecutions(vcc, scheduleData, executionCode);
    })
        .then(function (_) {
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Finishing function');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::DeleteBatchsScheduleExecutionsData] Function failed');
        return Promise.reject(err);
    });
}
function SaveProcessBatchSchedule(vcc, vccType, process, userId) {
    var res = false;
    process.scheduleData.firstExecutionDate = moment(process.scheduleData.firstExecutionDate).utc().toDate();
    process.scheduleData.startDate = moment(process.scheduleData.startDate).utc().toDate();
    process.scheduleData.endDate = process.scheduleData.endDate ? moment(process.scheduleData.endDate).utc().toDate() : null;
    var includeCallChannel = process.processFilters.filter(function (pf) { return pf.key === 'InteractionType' && pf.value === 'CALL'; }).length ? true : false;
    var includeTextChannel = process.processFilters.filter(function (pf) { return pf.key === 'InteractionType' && pf.value !== 'CALL'; }).length ? true : false;
    var executionDelay = 0;
    if (vccType === 'I6' && includeCallChannel)
        executionDelay = 1;
    if (includeTextChannel)
        executionDelay = 24;
    process.scheduleData.batchData = {
        clientId: process.clientId,
        description: process.description,
        type: process.type,
        language: process.language,
        dictionaryId: process.dictionaryId,
        environment: process.environment,
        environmentType: process.environmentType,
        executionDelay: executionDelay,
        samplePercentage: process.percentage,
        vccType: vccType
    };
    process.scheduleData.executionDelay = executionDelay;
    var executionList = [];
    process.scheduleData.executionList.map(function (el, i) {
        var execution = new schedule_model_1.ScheduleExecution();
        execution.batchScheduleId = process.scheduleData.id;
        execution.order = i + 1,
            execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = moment(el).toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = userId;
        executionList.push(execution);
    });
    process.scheduleData.executionList = executionList;
    return promises_shared_1.DummyPromise()
        .then(function (_) {
        return SaveBatchSchedule(vcc, process.scheduleData, userId);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_BATCH_SCHEDULE_SAVE_FAILED';
        var promiseArray = [];
        process.processFilters.map(function (pf) {
            promiseArray.push(mssql_service_1.MSSql.SaveProcessFilter(vcc, pf.processId, pf.key, pf.value, moment().utc().toDate(), pf.createdByUserId, moment().utc().toDate(), pf.lastModifiedByUserId));
        });
        return Promise.all(promiseArray);
    })
        .then(function (results) {
        logger.info('[BatchScheduleService::SaveProcessBatchSchedule] Finishing function');
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveProcessBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveProcessBatchSchedule = SaveProcessBatchSchedule;
function SaveEvaluationPlanBatchSchedule() {
    return promises_shared_1.DummyPromise()
        .then(function (_) {
    })
        .then(function (_) {
        logger.info('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Finishing function');
        return Promise.resolve(null);
    })
        .catch(function (err) {
        logger.error('[BatchScheduleService::SaveEvaluationPlanBatchSchedule] Function failed');
        return Promise.reject(err);
    });
}
exports.SaveEvaluationPlanBatchSchedule = SaveEvaluationPlanBatchSchedule;
function GenerateNewBatcScheduleExecutions(schedule, range, lastExecutionOrder) {
    if (!range)
        range = 7;
    if (!lastExecutionOrder)
        lastExecutionOrder = 0;
    var lastExecutionDate = moment(schedule.lastExecutionDate);
    var maxLastExecutionDate = moment(schedule.endDate ? schedule.endDate : moment().add(range, 'days'));
    var executionList = [];
    while (lastExecutionDate < maxLastExecutionDate) {
        switch (schedule.frequency) {
            case 'daily':
                lastExecutionDate.add(1, 'days');
                break;
            case 'weekly':
                lastExecutionDate.add(1, 'weeks');
                break;
            case 'monthly':
                lastExecutionDate.add(1, 'months');
                break;
            case 'custom_per_days':
                lastExecutionDate.add(schedule.daysInterval, 'days');
                break;
            case 'custom_per_hours':
                lastExecutionDate.add(schedule.hoursInterval, 'hours');
                break;
        }
        var execution = new schedule_model_1.ScheduleExecution();
        execution.vcc = schedule.vcc;
        execution.batchScheduleId = schedule.id;
        execution.order = ++lastExecutionOrder,
            execution.id = utilities_1.guid();
        execution.batchId = null;
        execution.executionDate = lastExecutionDate.toDate();
        execution.status = 'PENDING';
        execution.createdByUserId = schedule.createdByUserId;
        if (lastExecutionDate.isSameOrAfter(moment()) && lastExecutionDate.isBefore(maxLastExecutionDate))
            executionList.push(execution);
    }
    return executionList;
}
exports.GenerateNewBatcScheduleExecutions = GenerateNewBatcScheduleExecutions;
